@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -173,6 +173,18 @@ template<UnsignedInt dimensions> class TextureArray: public AbstractTexture {
          */
         explicit TextureArray(NoCreateT) noexcept: AbstractTexture{NoCreate, Implementation::textureArrayTarget<dimensions>()} {}
 
+        /** @brief Copying is not allowed */
+        TextureArray(const TextureArray<dimensions>&) = delete;
+
+        /** @brief Move constructor */
+        TextureArray(TextureArray<dimensions>&&) noexcept = default;
+
+        /** @brief Copying is not allowed */
+        TextureArray<dimensions>& operator=(const TextureArray<dimensions>&) = delete;
+
+        /** @brief Move assignment */
+        TextureArray<dimensions>& operator=(TextureArray<dimensions>&&) noexcept = default;
+
         #ifndef MAGNUM_TARGET_WEBGL
         /**
          * @brief Bind level of given texture layer to given image unit
@@ -387,20 +399,29 @@ template<UnsignedInt dimensions> class TextureArray: public AbstractTexture {
 
         #ifndef MAGNUM_TARGET_WEBGL
         /**
-         * @brief @copybrief Texture::setSRGBDecode()
+         * @brief @copybrief Texture::setSrgbDecode()
          * @return Reference to self (for method chaining)
          *
-         * See @ref Texture::setSRGBDecode() for more information.
+         * See @ref Texture::setSrgbDecode() for more information.
          * @requires_extension Extension @gl_extension{EXT,texture_sRGB_decode}
          * @requires_es_extension Extension @gl_extension{ANDROID,extension_pack_es31a} /
          *      @gl_extension2{EXT,texture_sRGB_decode,texture_sRGB_decode}
-         * @requires_gles SRGB decode is not available in WebGL.
+         * @requires_gles sRGB decode is not available in WebGL.
          */
-        TextureArray<dimensions>& setSRGBDecode(bool decode) {
-            AbstractTexture::setSRGBDecode(decode);
+        TextureArray<dimensions>& setSrgbDecode(bool decode) {
+            AbstractTexture::setSrgbDecode(decode);
             return *this;
         }
 
+        #ifdef MAGNUM_BUILD_DEPRECATED
+        /** @brief @copybrief setSrgbDecode()
+         * @deprecated Use @ref setSrgbDecode() instead.
+         */
+        CORRADE_DEPRECATED("use setSrgbDecode() instead") TextureArray<dimensions>& setSRGBDecode(bool decode) {
+            return setSrgbDecode(decode);
+        }
+        #endif
+
         /**
          * @brief @copybrief Texture::setSwizzle()
          * @return Reference to self (for method chaining)
