@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -29,10 +29,9 @@
  * @brief Class @ref Magnum::Platform::Sdl2Application, macro @ref MAGNUM_SDL2APPLICATION_MAIN()
  */
 
-#include <memory>
-#include <Corrade/Corrade.h>
 #include <Corrade/Containers/ArrayView.h>
 #include <Corrade/Containers/EnumSet.h>
+#include <Corrade/Containers/Pointer.h>
 
 #include "Magnum/Magnum.h"
 #include "Magnum/Tags.h"
@@ -46,7 +45,10 @@
 #ifdef CORRADE_TARGET_WINDOWS /* Windows version of SDL2 redefines main(), we don't want that */
 #define SDL_MAIN_HANDLED
 #endif
-#include <SDL.h>
+/* SDL.h includes the world, adding 50k LOC. We don't want that either. */
+#include <SDL_keycode.h>
+#include <SDL_mouse.h>
+#include <SDL_video.h>
 #include <SDL_scancode.h>
 
 #ifdef CORRADE_TARGET_WINDOWS_RT
@@ -54,6 +56,10 @@
 #include <wrl.h> /* For the WinMain entrypoint */
 #endif
 
+#ifndef DOXYGEN_GENERATING_OUTPUT
+union SDL_Event; /* for anyEvent() */
+#endif
+
 namespace Magnum { namespace Platform {
 
 namespace Implementation {
@@ -73,6 +79,14 @@ respective sections in @ref building-corrade-cross-emscripten "Corrade's" and
 on the [SDL2](http://www.libsdl.org) library (Emscripten has it built in) and
 is built if `WITH_SDL2APPLICATION` is enabled in CMake.
 
+@m_class{m-block m-success}
+
+@thirdparty This plugin makes use of the [SDL2](https://www.libsdl.org/)
+    library, released under the @m_class{m-label m-success} **zlib license**
+    ([license text](http://www.gzip.org/zlib/zlib_license.html),
+    [choosealicense.com](https://choosealicense.com/licenses/zlib/)).
+    Attribution is appreciated but not required.
+
 @section Platform-Sdl2Application-bootstrap Bootstrap application
 
 Fully contained base application using @ref Sdl2Application along with
@@ -228,13 +242,35 @@ If no other application header is included, this class is also aliased to
 @cpp Platform::Application @ce and the macro is aliased to @cpp MAGNUM_APPLICATION_MAIN() @ce
 to simplify porting.
 
+@subsection Platform-Sdl2Application-usage-posix POSIX specifics
+
+On POSIX systems, SDL by default intercepts the `SIGTERM` signal and generates
+an exit event for it, instead of doing the usual application exit. This would
+mean that if the application fails to set @ref ExitEvent::setAccepted() in an
+@ref exitEvent() override for some reason, pressing
+@m_class{m-label m-warning} **Ctrl** @m_class{m-label m-default} **C** would
+not terminate it either and you'd have to forcibly kill it instead. When using
+SDL >= 2.0.4, @ref Sdl2Application turns this behavior off, making
+@ref exitEvent() behave consistently with other application implementations
+such as @ref GlfwApplication. You can turn this behavior back on by enabling
+the [corresponding SDL hint](https://wiki.libsdl.org/SDL_HINT_NO_SIGNAL_HANDLERS)
+through an environment variable:
+
+@code{.sh}
+SDL_NO_SIGNAL_HANDLERS=1 ./your-app
+@endcode
+
+See also the [SDL Wiki](https://wiki.libsdl.org/SDL_EventType#SDL_QUIT) for
+details.
+
 @subsection Platform-Sdl2Application-usage-linux Linux specifics
 
 SDL by default attempts to disable compositing, which may cause ugly flickering
 for non-fullscreen apps (KWin, among others, is known to respect this setting).
 When using SDL >= 2.0.8, @ref Sdl2Application turns this behavior off, keeping
 the compositor running to avoid the flicker. You can turn this behavior back on
-by enabling the [corresponding SDL hint](https://wiki.libsdl.org/CategoryHints) through an environment variable:
+by enabling the [corresponding SDL hint](https://wiki.libsdl.org/CategoryHints)
+through an environment variable:
 
 @code{.sh}
 SDL_VIDEO_X11_NET_WM_BYPASS_COMPOSITOR=1 ./your-app
@@ -277,7 +313,8 @@ size and the framebuffer pixels will match display pixels 1:1. On displays that
 have different DPI, there are three possible scenarios, listed below. It's
 possible to fine tune the behavior either using extra parameters passed to
 @ref Configuration::setSize() or via the `--magnum-dpi-scaling` command-line
-option.
+option (or the equivalent @cb{.sh} $MAGNUM_DPI_SCALING @ce environment
+variable).
 
 -   Framebuffer DPI scaling. The window is created with exactly the requested
     size and all event coordinates are reported also relative to that size.
@@ -285,7 +322,8 @@ option.
     supported on macOS and iOS. See @ref platforms-macos-hidpi for details how
     to enable it. Equivalent to passing
     @ref Configuration::DpiScalingPolicy::Framebuffer to
-    @ref Configuration::setSize() or `framebuffer` on command line.
+    @ref Configuration::setSize() or `framebuffer` via command line /
+    environment.
 -   Virtual DPI scaling. Scales the window based on DPI scaling setting in the
     system. For example if a 800x600 window is requested and DPI scaling is set
     to 200%, the resulting window will have 1600x1200 pixels. The backing
@@ -303,15 +341,16 @@ option.
     This is supported on Linux, Windows, all mobile platforms except iOS and
     Emscripten. Equivalent to passing
     @ref Configuration::DpiScalingPolicy::Physical to
-    @ref Configuration::setSize() or `physical` on command line.
+    @ref Configuration::setSize() or `physical` via command line / environment.
 
 Besides the above, it's possible to supply a custom DPI scaling value to
 @ref Configuration::setSize() or the `--magnum-dpi-scaling` command-line
-option. Using `--magnum-dpi-scaling &lt;number&gt;` will make the scaling
-same in both directions, with `--magnum-dpi-scaling "<horizontal> <vertical>"`
-the scaling will be different in each direction. On desktop systems custom DPI
-scaling value will affect physical window size (with the content being scaled),
-on mobile and web it will affect sharpness of the contents.
+option (or environment variable). Using `--magnum-dpi-scaling &lt;number&gt;`
+will make the scaling same in both directions, with
+`--magnum-dpi-scaling "<horizontal> <vertical>"` the scaling will be different
+in each direction. On desktop systems custom DPI scaling value will affect
+physical window size (with the content being scaled), on mobile and web it will
+affect sharpness of the contents.
 
 The default is depending on the platform:
 
@@ -367,6 +406,7 @@ class Sdl2Application {
         #ifdef MAGNUM_TARGET_GL
         class GLConfiguration;
         #endif
+        class ExitEvent;
         class ViewportEvent;
         class InputEvent;
         class KeyEvent;
@@ -429,14 +469,6 @@ class Sdl2Application {
          */
         explicit Sdl2Application(const Arguments& arguments, NoCreateT);
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
-        /**
-         * @brief @copybrief Sdl2Application(const Arguments&, NoCreateT)
-         * @deprecated Use @ref Sdl2Application(const Arguments&, NoCreateT) instead.
-         */
-        CORRADE_DEPRECATED("use Sdl2Application(const Arguments&, NoCreateT) instead") explicit Sdl2Application(const Arguments& arguments, std::nullptr_t): Sdl2Application{arguments, NoCreate} {}
-        #endif
-
         /** @brief Copying is not allowed */
         Sdl2Application(const Sdl2Application&) = delete;
 
@@ -480,7 +512,8 @@ class Sdl2Application {
         /**
          * @brief Underlying window handle
          *
-         * Use in case you need to call SDL functionality directly.
+         * Use in case you need to call SDL functionality directly. Returns
+         * @cpp nullptr @ce in case the window was not created yet.
          * @note Not available in @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten".
          */
         SDL_Window* window() { return _window; }
@@ -656,7 +689,6 @@ class Sdl2Application {
         void setContainerCssClass(const std::string& cssClass);
         #endif
 
-    protected:
         /**
          * @brief Swap buffers
          *
@@ -705,22 +737,7 @@ class Sdl2Application {
          */
         void redraw() { _flags |= Flag::Redraw; }
 
-    #ifdef DOXYGEN_GENERATING_OUTPUT
-    protected:
-    #else
     private:
-    #endif
-        /**
-         * @brief Tick event
-         *
-         * If implemented, this function is called periodically after
-         * processing all input events and before draw event even though there
-         * might be no input events and redraw is not requested. Useful e.g.
-         * for asynchronous task polling. Use @ref setMinimalLoopPeriod()/
-         * @ref setSwapInterval() to control main loop frequency.
-         */
-        virtual void tickEvent();
-
         /**
          * @brief Viewport event
          *
@@ -801,11 +818,7 @@ class Sdl2Application {
          */
         void setMouseLocked(bool enabled);
 
-    #ifdef DOXYGEN_GENERATING_OUTPUT
-    protected:
-    #else
     private:
-    #endif
         /**
          * @brief Mouse press event
          *
@@ -894,11 +907,7 @@ class Sdl2Application {
          */
         void setTextInputRect(const Range2Di& rect);
 
-    #ifdef DOXYGEN_GENERATING_OUTPUT
-    protected:
-    #else
     private:
-    #endif
         /**
          * @brief Text input event
          *
@@ -916,17 +925,59 @@ class Sdl2Application {
 
         /*@}*/
 
+        /** @{ @name Special events */
+
+        /**
+         * @brief Exit event
+         *
+         * If implemented, it allows the application to react to an application
+         * exit (for example to save its internal state) and suppress it as
+         * well (for example to show a exit confirmation dialog). The default
+         * implementation calls @ref ExitEvent::setAccepted() on @p event,
+         * which tells the application that it's safe to exit.
+         *
+         * SDL has special behavior on POSIX systems regarding `SIGINT` and
+         * `SIGTERM` handling, see @ref Platform-Sdl2Application-usage-posix
+         * for more information.
+         */
+        virtual void exitEvent(ExitEvent& event);
+
+        /**
+         * @brief Tick event
+         *
+         * If implemented, this function is called periodically after
+         * processing all input events and before draw event even though there
+         * might be no input events and redraw is not requested. Useful e.g.
+         * for asynchronous task polling. Use @ref setMinimalLoopPeriod()/
+         * @ref setSwapInterval() to control main loop frequency.
+         */
+        virtual void tickEvent();
+
+        /**
+         * @brief Any event
+         *
+         * Called in case a SDL event is not handled by any other event
+         * functions above.
+         * @see @ref ViewportEvent::event(), @ref InputEvent::event(),
+         *      @ref MultiGestureEvent::event(), @ref TextInputEvent::event(),
+         *      @ref TextEditingEvent::event(), @ref ExitEvent::event()
+         */
+        virtual void anyEvent(SDL_Event& event);
+
+        /*@}*/
+
     private:
         enum class Flag: UnsignedByte {
             Redraw = 1 << 0,
             VSyncEnabled = 1 << 1,
             NoTickEvent = 1 << 2,
+            NoAnyEvent = 1 << 3,
             #ifndef CORRADE_TARGET_EMSCRIPTEN
-            Exit = 1 << 3
+            Exit = 1 << 4
             #endif
             #ifdef CORRADE_TARGET_EMSCRIPTEN
-            TextInputActive = 1 << 4,
-            Resizable = 1 << 5
+            TextInputActive = 1 << 5,
+            Resizable = 1 << 6
             #endif
         };
 
@@ -953,7 +1004,7 @@ class Sdl2Application {
         #else
         SDL_Surface* _glContext{};
         #endif
-        std::unique_ptr<Platform::GLContext> _context;
+        Containers::Pointer<Platform::GLContext> _context;
         #endif
 
         Flags _flags;
@@ -1601,6 +1652,54 @@ class Sdl2Application::Configuration {
         #endif
 };
 
+/**
+@brief Exit event
+
+@see @ref exitEvent()
+*/
+class Sdl2Application::ExitEvent {
+    public:
+        /** @brief Copying is not allowed */
+        ExitEvent(const ExitEvent&) = delete;
+
+        /** @brief Moving is not allowed */
+        ExitEvent(ExitEvent&&) = delete;
+
+        /** @brief Copying is not allowed */
+        ExitEvent& operator=(const ExitEvent&) = delete;
+
+        /** @brief Moving is not allowed */
+        ExitEvent& operator=(ExitEvent&&) = delete;
+
+        /** @brief Whether the event is accepted */
+        bool isAccepted() const { return _accepted; }
+
+        /**
+         * @brief Set event as accepted
+         *
+         * If the event is ignored (i.e., not set as accepted) in
+         * @ref exitEvent(), the application won't exit. Default implementation
+         * of @ref exitEvent() accepts the event.
+         */
+        void setAccepted(bool accepted = true) { _accepted = accepted; }
+
+        /**
+         * @brief Underlying SDL event
+         *
+         * Of type `SDL_QUIT`.
+         * @see @ref Sdl2Application::anyEvent()
+         */
+        const SDL_Event& event() const { return _event; }
+
+    private:
+        friend Sdl2Application;
+
+        explicit ExitEvent(const SDL_Event& event): _event(event), _accepted(false) {}
+
+        const SDL_Event& _event;
+        bool _accepted;
+};
+
 /**
 @brief Viewport event
 
@@ -1608,6 +1707,18 @@ class Sdl2Application::Configuration {
 */
 class Sdl2Application::ViewportEvent {
     public:
+        /** @brief Copying is not allowed */
+        ViewportEvent(const ViewportEvent&) = delete;
+
+        /** @brief Moving is not allowed */
+        ViewportEvent(ViewportEvent&&) = delete;
+
+        /** @brief Copying is not allowed */
+        ViewportEvent& operator=(const ViewportEvent&) = delete;
+
+        /** @brief Moving is not allowed */
+        ViewportEvent& operator=(ViewportEvent&&) = delete;
+
         /**
          * @brief Window size
          *
@@ -1639,13 +1750,36 @@ class Sdl2Application::ViewportEvent {
          */
         Vector2 dpiScaling() const { return _dpiScaling; }
 
+        #ifndef CORRADE_TARGET_EMSCRIPTEN
+        /**
+         * @brief Underlying SDL event
+         *
+         * Of type `SDL_WINDOWEVENT`.
+         * @note Not available in @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten".
+         * @see @ref Sdl2Application::anyEvent()
+         */
+        const SDL_Event& event() const { return _event; }
+        #endif
+
     private:
         friend Sdl2Application;
 
-        explicit ViewportEvent(const Vector2i& windowSize, const Vector2i& framebufferSize, const Vector2& dpiScaling): _windowSize{windowSize}, _framebufferSize{framebufferSize}, _dpiScaling{dpiScaling} {}
+        explicit ViewportEvent(
+            #ifndef CORRADE_TARGET_EMSCRIPTEN
+            const SDL_Event& event,
+            #endif
+            const Vector2i& windowSize, const Vector2i& framebufferSize, const Vector2& dpiScaling):
+                #ifndef CORRADE_TARGET_EMSCRIPTEN
+                _event(event),
+                #endif
+                _windowSize{windowSize}, _framebufferSize{framebufferSize}, _dpiScaling{dpiScaling} {}
 
-        Vector2i _windowSize, _framebufferSize;
-        Vector2 _dpiScaling;
+        #ifndef CORRADE_TARGET_EMSCRIPTEN
+        const SDL_Event& _event;
+        #endif
+        const Vector2i _windowSize;
+        const Vector2i _framebufferSize;
+        const Vector2 _dpiScaling;
 };
 
 /**
@@ -1736,14 +1870,26 @@ class Sdl2Application::InputEvent {
          */
         void setAccepted(bool accepted = true) { _accepted = accepted; }
 
+        /**
+         * @brief Underlying SDL event
+         *
+         * Of type `SDL_KEYDOWN` / `SDL_KEYUP` for @ref KeyEvent,
+         * `SDL_MOUSEBUTTONUP` / `SDL_MOUSEBUTTONDOWN` for @ref MouseEvent,
+         * `SDL_MOUSEWHEEL` for @ref MouseScrollEvent and `SDL_MOUSEMOTION` for
+         * @ref MouseMoveEvent.
+         * @see @ref Sdl2Application::anyEvent()
+         */
+        const SDL_Event& event() const { return _event; }
+
     #ifndef DOXYGEN_GENERATING_OUTPUT
     protected:
-        explicit InputEvent(): _accepted(false) {}
+        explicit InputEvent(const SDL_Event& event): _event(event), _accepted(false) {}
 
         ~InputEvent() = default;
     #endif
 
     private:
+        const SDL_Event& _event;
         bool _accepted;
 };
 
@@ -1963,7 +2109,7 @@ class Sdl2Application::KeyEvent: public Sdl2Application::InputEvent {
         bool isRepeated() const { return _repeated; }
 
     private:
-        explicit KeyEvent(Key key, Modifiers modifiers, bool repeated): _key{key}, _modifiers{modifiers}, _repeated{repeated} {}
+        explicit KeyEvent(const SDL_Event& event, Key key, Modifiers modifiers, bool repeated): InputEvent{event}, _key{key}, _modifiers{modifiers}, _repeated{repeated} {}
 
         const Key _key;
         const Modifiers _modifiers;
@@ -1995,20 +2141,6 @@ class Sdl2Application::MouseEvent: public Sdl2Application::InputEvent {
 
             /** Second extra button (e.g. wheel right) */
             X2 = SDL_BUTTON_X2,
-
-            #ifdef MAGNUM_BUILD_DEPRECATED
-            /**
-             * Wheel up
-             * @deprecated Use @ref MouseScrollEvent and @ref mouseScrollEvent() instead.
-             */
-            WheelUp CORRADE_DEPRECATED_ENUM("use mouseScrollEvent() and MouseScrollEvent instead") = SDL_BUTTON_X2 + 1,
-
-            /**
-             * Wheel down
-             * @deprecated Use @ref MouseScrollEvent and @ref mouseScrollEvent() instead.
-             */
-            WheelDown CORRADE_DEPRECATED_ENUM("use mouseScrollEvent() and MouseScrollEvent instead") = SDL_BUTTON_X2 + 2
-            #endif
         };
 
         /** @brief Button */
@@ -2034,11 +2166,11 @@ class Sdl2Application::MouseEvent: public Sdl2Application::InputEvent {
         Modifiers modifiers();
 
     private:
-        explicit MouseEvent(Button button, const Vector2i& position
+        explicit MouseEvent(const SDL_Event& event, Button button, const Vector2i& position
             #ifndef CORRADE_TARGET_EMSCRIPTEN
             , Int clickCount
             #endif
-            ): _button{button}, _position{position},
+            ): InputEvent{event}, _button{button}, _position{position},
             #ifndef CORRADE_TARGET_EMSCRIPTEN
             _clickCount{clickCount},
             #endif
@@ -2107,7 +2239,7 @@ class Sdl2Application::MouseMoveEvent: public Sdl2Application::InputEvent {
         Modifiers modifiers();
 
     private:
-        explicit MouseMoveEvent(const Vector2i& position, const Vector2i& relativePosition, Buttons buttons): _position{position}, _relativePosition{relativePosition}, _buttons{buttons}, _modifiersLoaded{false} {}
+        explicit MouseMoveEvent(const SDL_Event& event, const Vector2i& position, const Vector2i& relativePosition, Buttons buttons): InputEvent{event}, _position{position}, _relativePosition{relativePosition}, _buttons{buttons}, _modifiersLoaded{false} {}
 
         const Vector2i _position, _relativePosition;
         const Buttons _buttons;
@@ -2142,7 +2274,7 @@ class Sdl2Application::MouseScrollEvent: public Sdl2Application::InputEvent {
         Modifiers modifiers();
 
     private:
-        explicit MouseScrollEvent(const Vector2& offset): _offset{offset}, _positionLoaded{false}, _modifiersLoaded{false} {}
+        explicit MouseScrollEvent(const SDL_Event& event, const Vector2& offset): InputEvent{event}, _offset{offset}, _positionLoaded{false}, _modifiersLoaded{false} {}
 
         const Vector2 _offset;
         bool _positionLoaded;
@@ -2210,13 +2342,22 @@ class Sdl2Application::MultiGestureEvent {
          */
         Int fingerCount() const { return _fingerCount; }
 
-    private:
-        explicit MultiGestureEvent(const Vector2& center, Float relativeRotation, Float relativeDistance, Int fingerCount): _center{center}, _relativeRotation{relativeRotation}, _relativeDistance{relativeDistance}, _fingerCount{fingerCount}, _accepted{false} {}
+        /**
+         * @brief Underlying SDL event
+         *
+         * Of type `SDL_MULTIGESTURE`.
+         * @see @ref Sdl2Application::anyEvent()
+         */
+        const SDL_Event& event() const { return _event; }
 
-        Vector2 _center;
-        Float _relativeRotation,
-            _relativeDistance;
-        Int _fingerCount;
+    private:
+        explicit MultiGestureEvent(const SDL_Event& event, const Vector2& center, Float relativeRotation, Float relativeDistance, Int fingerCount): _event(event), _center{center}, _relativeRotation{relativeRotation}, _relativeDistance{relativeDistance}, _fingerCount{fingerCount}, _accepted{false} {}
+
+        const SDL_Event& _event;
+        const Vector2 _center;
+        const Float _relativeRotation;
+        const Float _relativeDistance;
+        const Int _fingerCount;
         bool _accepted;
 };
 
@@ -2257,10 +2398,19 @@ class Sdl2Application::TextInputEvent {
         /** @brief Input text in UTF-8 */
         Containers::ArrayView<const char> text() const { return _text; }
 
-    private:
-        explicit TextInputEvent(Containers::ArrayView<const char> text): _text{text}, _accepted{false} {}
+        /**
+         * @brief Underlying SDL event
+         *
+         * Of type `SDL_TEXTINPUT`.
+         * @see @ref Sdl2Application::anyEvent()
+         */
+        const SDL_Event& event() const { return _event; }
 
-        Containers::ArrayView<const char> _text;
+    private:
+        explicit TextInputEvent(const SDL_Event& event, Containers::ArrayView<const char> text): _event(event), _text{text}, _accepted{false} {}
+
+        const SDL_Event& _event;
+        const Containers::ArrayView<const char> _text;
         bool _accepted;
 };
 
@@ -2307,11 +2457,21 @@ class Sdl2Application::TextEditingEvent {
         /** @brief Number of characters to edit from the start point */
         Int length() const { return _length; }
 
-    private:
-        explicit TextEditingEvent(Containers::ArrayView<const char> text, Int start, Int length): _text{text}, _start{start}, _length{length}, _accepted{false} {}
+        /**
+         * @brief Underlying SDL event
+         *
+         * Of type `SDL_TEXTEDITING`.
+         * @see @ref Sdl2Application::anyEvent()
+         */
+        const SDL_Event& event() const { return _event; }
 
-        Containers::ArrayView<const char> _text;
-        Int _start, _length;
+    private:
+        explicit TextEditingEvent(const SDL_Event& event, Containers::ArrayView<const char> text, Int start, Int length): _event(event), _text{text}, _start{start}, _length{length}, _accepted{false} {}
+
+        const SDL_Event& _event;
+        const Containers::ArrayView<const char> _text;
+        const Int _start;
+        const Int _length;
         bool _accepted;
 };
 
