@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -216,6 +216,17 @@ template<UnsignedInt dimensions> class ImageData {
          */
         template<class T> explicit ImageData(CompressedPixelStorage storage, T format, const VectorTypeFor<dimensions, Int>& size, Containers::Array<char>&& data, const void* importerState = nullptr) noexcept;
 
+        /**
+         * @brief Construct from existing data with attached importer state
+         *
+         * Useful in cases where importer plugins proxy image loading through
+         * other importers but want to attach its own importer state to the
+         * imported data. Importer state from the @p other object is replaced
+         * with @p importerState, data ownership is transferred and everything
+         * else stays the same.
+         */
+        explicit ImageData(ImageData<dimensions>&& other, const void* importerState) noexcept;
+
         /** @brief Copying is not allowed */
         ImageData(const ImageData<dimensions>&) = delete;
 
@@ -292,7 +303,7 @@ template<UnsignedInt dimensions> class ImageData {
          *
          * @deprecated Cast @ref formatExtra() to @ref GL::PixelType instead.
          */
-        CORRADE_DEPRECATED("cast formatExtra() to GL::PixelType instead") GL::PixelType type() const { return GL::PixelType(formatExtra()); }
+        CORRADE_DEPRECATED("cast formatExtra() to GL::PixelType instead") GL::PixelType type() const { return GL::PixelType(formatExtra()); } /* LCOV_EXCL_LINE */
         #endif
 
         /**
@@ -346,24 +357,12 @@ template<UnsignedInt dimensions> class ImageData {
          *
          * @see @ref release()
          */
-        Containers::ArrayView<char> data()
-            #ifndef CORRADE_GCC47_COMPATIBILITY
-            &
-            #endif
-            { return _data; }
-        #ifndef CORRADE_GCC47_COMPATIBILITY
+        Containers::ArrayView<char> data() & { return _data; }
         Containers::ArrayView<char> data() && = delete; /**< @overload */
-        #endif
 
         /** @overload */
-        Containers::ArrayView<const char> data() const
-            #ifndef CORRADE_GCC47_COMPATIBILITY
-            &
-            #endif
-            { return _data; }
-        #ifndef CORRADE_GCC47_COMPATIBILITY
+        Containers::ArrayView<const char> data() const & { return _data; }
         Containers::ArrayView<const char> data() const && = delete; /**< @overload */
-        #endif
 
         /** @overload */
         template<class T> T* data() {
